/* 
This code controls the RGB-LED tree published on instructable. It is written for a Wemos D1 mini. If other microcontrollers are used adjustments might be necesarry.
This code makes use of the FastLED library further Informations can be found on http://fastled.io/
The FastLED library can be downloaded from https://github.com/FastLED/FastLED
*/

#include <FastLED.h>
//Define the Pins on the Wemos to be used
#define REDPIN D7
#define GREENPIN D6
#define BLUEPIN D5
#define BUTTONPIN D3

//Color Mode
#define RGB_MODE_OFF 0
#define RGB_MODE_RAINBOW 1
#define RGB_Mode_STATIC 2
uint8_t g_rgb_mode = RGB_MODE_RAINBOW;
uint32_t g_last_colorchange = 0;
/*
Defines how long one specific color is shown in milliseconds in Rainbow mode. 
One full cycle is 255 different colors. With 100ms one full cycle will therefore take 25.5 seconds
*/
uint32_t g_wait_intervall = 100; 
uint8_t g_current_button_state = HIGH;
uint8_t g_previous_button_state = LOW;
uint32_t g_debounce_delay = 100; //Time in milliseconds that a new button press is ignored after previous one.
uint32_t g_last_debounce_time;
uint8_t g_fixed_hue = 0;

//Sets PWM-Signal for the 3 colors
void AnalogWriteColor( const CRGB& rgb)
{
  analogWrite(REDPIN,   rgb.r );
  analogWrite(GREENPIN, rgb.g );
  analogWrite(BLUEPIN,  rgb.b );
}

//Flashes the Colors Red, Green, Blue and White.
void colorFlash()
{
  AnalogWriteColor( CRGB::Red );   delay(500);
  AnalogWriteColor( CRGB::Green ); delay(500);
  AnalogWriteColor( CRGB::Blue );  delay(500);
  AnalogWriteColor( CRGB::White ); delay(500);
  AnalogWriteColor( CRGB::Black ); delay(500);
}

//Cycles the static colors in static mode. To change the colors that are beeing cycled change the numbers after g_fixed_hue and in the case below
void static_colorchange()
{
  switch(g_fixed_hue){
    case 0:
      g_fixed_hue = 42;
      break;
    case 42:
      g_fixed_hue = 85;
      break;
    case 85:
      g_fixed_hue = 127;
      break;
    case 127:
      g_fixed_hue = 170;
      break;
    case 170:
      g_fixed_hue = 212;
      break;
    case 212:
      g_fixed_hue = 0;
      break;
    default:
      colorFlash();
      break;
  }
  AnalogWriteColor(CHSV( g_fixed_hue, 255, 255));
}

//Flashes white
void flash(uint8_t number_of_flashes)
{
  for (uint8_t i = 0; i<number_of_flashes; i++)
  {
  AnalogWriteColor( CRGB::Black ); delay(500);
  AnalogWriteColor( CRGB::White ); delay(500);
  }
  AnalogWriteColor( CRGB::Black ); delay(500);
}

//Checks if button is pressed and how long and depending on that changes modes or colors
void checkButtonPress()
{
  g_current_button_state = digitalRead(BUTTONPIN);
  if (HIGH == g_previous_button_state && LOW == g_current_button_state){
    uint32_t last_button_pressed_time = millis();
    while (digitalRead(BUTTONPIN)==LOW){
      //do Nothing
    }
    uint32_t button_hold_time = millis() - last_button_pressed_time;
    if (millis() - g_last_debounce_time <= g_debounce_delay){
      return;
    }
    g_last_debounce_time =  millis();
    if (button_hold_time >= 500){
      g_rgb_mode++;
      if (g_rgb_mode > 2){
        g_rgb_mode = 0;
      }
      flash(g_rgb_mode);
    }
    else if (RGB_Mode_STATIC == g_rgb_mode){
      static_colorchange();
    }
    else {
      colorFlash();
    }
  }
  g_previous_button_state = g_current_button_state;
}


//Sets Red, green and blue pin to output to be used later, sets the buttonpin to input for the button and activates internal pullup.
//Without internal pullup butten might be detected as pressed forever. In this case external pullup is needed.
void setup() 
{
  pinMode(REDPIN,   OUTPUT);
  pinMode(GREENPIN, OUTPUT);
  pinMode(BLUEPIN,  OUTPUT);
  pinMode(BUTTONPIN, INPUT_PULLUP);
  colorFlash();
}

//Cycles through all colors of the hue spectrum. How fast this happens can be defined at the beginnen of the code with the variable g_wait_intervall
void rainbowcolor()
{
  if ( millis() - g_last_colorchange >= g_wait_intervall){
  static uint8_t hue;
  hue = hue + 1;
  // Use FastLED automatic HSV->RGB conversion
  AnalogWriteColor( CHSV( hue, 255, 255) );
  g_last_colorchange = millis();
  }
  
}

/*
Main part of Programm that runs repeatadly.
Depending on Mode set in function checkButtonPress or default defined in beginning of code calls the corresponding function and then checks for button press.
Modes are: Rainbow, Fixed Color, OFF; If an error occurs the tree will flash as in the beginning
*/
void loop() 
{ 
  switch(g_rgb_mode)
  {
  case RGB_MODE_OFF:
    AnalogWriteColor(CRGB::Black);
    break;
  case RGB_MODE_RAINBOW:
    rainbowcolor();
    break;
  case RGB_Mode_STATIC:
    AnalogWriteColor(CHSV( g_fixed_hue, 255, 255));
    break;
  default:
    colorFlash();
    break;
  }
  checkButtonPress();
}